@@ -44,6 +44,7 @@ typedef array_t(struct endpoint*) endpoint_array_t;
 
 struct net_tcp_param {
 	uint64_t in_idle_timeout;
+	uint64_t tls_handshake_timeout;
 };
 
 struct tls_session_ticket_ctx;
@@ -54,9 +55,10 @@ struct network {
 	map_t tls_client_params;
 	struct tls_session_ticket_ctx *tls_session_ticket_ctx;
 	struct net_tcp_param tcp;
+	int tcp_backlog;
 };
 
-void network_init(struct network *net, uv_loop_t *loop);
+void network_init(struct network *net, uv_loop_t *loop, int tcp_backlog);
 void network_deinit(struct network *net);
 int network_listen_fd(struct network *net, int fd, bool use_tls);
 int network_listen(struct network *net, const char *addr, uint16_t port, uint32_t flags);
@@ -64,3 +66,5 @@ int network_close(struct network *net, const char *addr, uint16_t port);
 int network_set_tls_cert(struct network *net, const char *cert);
 int network_set_tls_key(struct network *net, const char *key);
 void network_new_hostname(struct network *net, struct engine *engine);
+int network_set_bpf(struct network *net, int bpf_fd);
+void network_clear_bpf(struct network *net);
