@@ -493,16 +493,21 @@ ssize_t tls_process_input_data(struct session *s, const uint8_t *buf, ssize_t nr
 			kr_log_verbose("[%s] TLS rehandshake with %s has started\n",
 				       logstring,  kr_straddr(peer));
 			tls_set_hs_state(tls_p, TLS_HS_IN_PROGRESS);
+			int err = kr_ok();
 			while (tls_p->handshake_state <= TLS_HS_IN_PROGRESS) {
-				int err = tls_handshake(tls_p, tls_p->handshake_cb);
+				err = tls_handshake(tls_p, tls_p->handshake_cb);
 				if (err == kr_error(EAGAIN)) {
 					break;
 				} else if (err != kr_ok()) {
 					return err;
 				}
 			}
-			/* Wait for more data */
-			break;
+			if (err == kr_error(EAGAIN)) {
+				/* pull function is out of data */
+				break;
+			}
+			/* There are can be data available, check it. */
+			continue;
 		} else if (count < 0) {
 			kr_log_verbose("[%s] gnutls_record_recv failed: %s (%zd)\n",
 				     logstring, gnutls_strerror_name(count), count);
@@ -514,8 +519,21 @@ ssize_t tls_process_input_data(struct session *s, const uint8_t *buf, ssize_t nr
 		wire_buf += count;
 		wire_buf_size -= count;
 		submitted += count;
+		if (wire_buf_size == 0 && tls_p->consumed != tls_p->nread) {
+			/* session buffer is full
+			 * whereas not all the data were consumed */
+			return kr_error(ENOSPC);
+		}
+	}
+	/* Here all data must be consumed. */
+	if (tls_p->consumed != tls_p->nread) {
+		/* Something went wrong, better return error.
+		 * This is most probably due to gnutls_record_recv() did not
+		 * consume all available network data by calling kres_gnutls_pull().
+		 * TODO assess the need for buffering of data amount.
+		 */
+		return kr_error(ENOSPC);
 	}
-	assert(tls_p->consumed == tls_p->nread);
 	return submitted;
 }
 
@@ -1061,8 +1079,8 @@ static int client_verify_certificate(gnutls_session_t tls_session)
 		}
 
 		DEBUG_MSG("[tls_client] received pin  : %s\n", cert_pin);
-		for (size_t i = 0; i < ctx->params->pins.len; ++i) {
-			const char *pin = ctx->params->pins.at[i];
+		for (size_t j = 0; j < ctx->params->pins.len; ++j) {
+			const char *pin = ctx->params->pins.at[j];
 			bool match = (strcmp(cert_pin, pin) == 0);
 			DEBUG_MSG("[tls_client] configured pin: %s matches? %s\n",
 				  pin, match ? "yes" : "no");
@@ -1188,6 +1206,20 @@ void tls_client_ctx_free(struct tls_client_ctx_t *ctx)
 	free (ctx);
 }
 
+int  tls_pull_timeout_func(gnutls_transport_ptr_t h, unsigned int ms)
+{
+	struct tls_common_ctx *t = (struct tls_common_ctx *)h;
+	assert(t != NULL);
+	ssize_t	avail = t->nread - t->consumed;
+	DEBUG_MSG("[%s] timeout check: available: %zu\n",
+		  t->client_side ? "tls_client" : "tls", avail);
+	if (avail <= 0) {
+		errno = EAGAIN;
+		return -1;
+	}
+	return avail;
+}
+
 int tls_client_connect_start(struct tls_client_ctx_t *client_ctx,
 			     struct session *session,
 			     tls_handshake_cb handshake_cb)
@@ -1202,6 +1234,7 @@ int tls_client_connect_start(struct tls_client_ctx_t *client_ctx,
 
 	gnutls_session_set_ptr(ctx->tls_session, client_ctx);
 	gnutls_handshake_set_timeout(ctx->tls_session, ctx->worker->engine->net.tcp.tls_handshake_timeout);
+	gnutls_transport_set_pull_timeout_function(ctx->tls_session, tls_pull_timeout_func);
 	session_tls_set_client_ctx(session, client_ctx);
 	ctx->handshake_cb = handshake_cb;
 	ctx->handshake_state = TLS_HS_IN_PROGRESS;
@@ -1215,8 +1248,7 @@ int tls_client_connect_start(struct tls_client_ctx_t *client_ctx,
 
 	/* See https://www.gnutls.org/manual/html_node/Asynchronous-operation.html */
 	while (ctx->handshake_state <= TLS_HS_IN_PROGRESS) {
-		/* Don't pass the handshake callback as the connection isn't registered yet. */
-		int ret = tls_handshake(ctx, NULL);
+		int ret = tls_handshake(ctx, handshake_cb);
 		if (ret != kr_ok()) {
 			return ret;
 		}
