@@ -22,8 +22,8 @@ source_suffix = '.rst'
 master_doc = 'index'
 
 # General information about the project.
-project = u'Knot DNS Resolver'
-copyright = u'2014-2017 CZ.NIC labs'
+project = u'Knot Resolver'
+copyright = u'2014-2018 CZ.NIC labs'
 version = { k[0][0]: k[0][1] for k in filter(None, [re.findall(r'(MAJOR|MINOR|PATCH) := ([0-9]+)',line) for line in open('../config.mk')])}
 version = '%s.%s.%s' % (version['MAJOR'], version['MINOR'], version['PATCH'])
 release = version
@@ -56,7 +56,7 @@ html_theme_path = [sphinx_rtd_theme.get_html_theme_path()]
 # Grouping the document tree into LaTeX files. List of tuples
 # (source start file, target name, title, author, documentclass [howto/manual]).
 latex_documents = [
-  ('index', 'format.tex', u'Knot DNS Resolver',
+  ('index', 'format.tex', u'Knot Resolver',
    u'CZ.NIC Labs', 'manual'),
 ]
 
