@@ -130,10 +130,20 @@ int kr_cache_open(struct kr_cache *cache, const struct kr_cdb_api *api, struct k
 	cache->ttl_max = KR_CACHE_DEFAULT_TTL_MAX;
 	/* Check cache ABI version */
 	kr_cache_make_checkpoint(cache);
-	(void) assert_right_version(cache);
+	(void)assert_right_version(cache);
+
+	char *fpath;
+	ret = asprintf(&fpath, "%s/data.mdb", opts->path);
+	if (ret > 0) {
+		kr_cache_emergency_file_to_remove = fpath;
+	} else {
+		assert(false); /* non-critical, but still */
+	}
 	return 0;
 }
 
+const char *kr_cache_emergency_file_to_remove = NULL;
+
 
 #define cache_isvalid(cache) ((cache) && (cache)->api && (cache)->db)
 
@@ -143,6 +153,8 @@ void kr_cache_close(struct kr_cache *cache)
 		cache_op(cache, close);
 		cache->db = NULL;
 	}
+	free(/*const-cast*/(char*)kr_cache_emergency_file_to_remove);
+	kr_cache_emergency_file_to_remove = NULL;
 }
 
 int kr_cache_sync(struct kr_cache *cache)
@@ -303,7 +315,8 @@ int cache_peek(kr_layer_t *ctx, knot_pkt_t *pkt)
 	struct kr_query *qry = req->current_query;
 	/* We first check various exit-conditions and then call the _real function. */
 
-	if (ctx->state & (KR_STATE_FAIL|KR_STATE_DONE) || qry->flags.NO_CACHE
+	if (!kr_cache_is_open(&req->ctx->cache)
+	    || ctx->state & (KR_STATE_FAIL|KR_STATE_DONE) || qry->flags.NO_CACHE
 	    || (qry->flags.CACHE_TRIED && !qry->stale_cb)
 	    || !check_rrtype(qry->stype, qry) /* LATER: some other behavior for some of these? */
 	    || qry->sclass != KNOT_CLASS_IN) {
@@ -346,7 +359,8 @@ int cache_stash(kr_layer_t *ctx, knot_pkt_t *pkt)
 
 	/* Note: we cache even in KR_STATE_FAIL.  For example,
 	 * BOGUS answer can go to +cd cache even without +cd request. */
-	if (!qry || qry->flags.CACHED || !check_rrtype(knot_pkt_qtype(pkt), qry)
+	if (!kr_cache_is_open(cache) || !qry
+	    || qry->flags.CACHED || !check_rrtype(knot_pkt_qtype(pkt), qry)
 	    || qry->sclass != KNOT_CLASS_IN) {
 		return ctx->state;
 	}
@@ -602,7 +616,8 @@ static int stash_rrarray_entry(ranked_rr_array_t *arr, int arr_i,
 	ssize_t written = stash_rrset(cache, qry, rr, rr_sigs, qry->timestamp.tv_sec,
 					entry->rank, nsec_pmap, has_optout);
 	if (written < 0) {
-		kr_log_error("[%5hu][cach] stash failed, ret = %d\n", qry->id, ret);
+		kr_log_error("[%05u.%02u][cach] stash failed, ret = %d\n", qry->request->uid,
+			     qry->uid, ret);
 		return (int) written;
 	}
 
