@@ -21,7 +21,15 @@ typedef struct lru_group lru_group_t;
 
 struct lru_item {
 	uint16_t key_len, val_len; /**< Two bytes should be enough for our purposes. */
-	char data[];               /**< Place for both key and value. */
+	char data[];
+	/**< Place for both key and value.
+	 *
+	 * We use "char" to satisfy the C99+ aliasing rules.
+	 * See C99 section 6.5 Expressions, paragraph 7.
+	 * Any type can be accessed through char-pointer,
+	 * so we can use a common struct definition
+	 * for all types being held.
+	 */
 };
 
 /** @internal Compute offset of value in struct lru_item. */
@@ -157,8 +165,15 @@ KR_EXPORT void * lru_get_impl(struct lru *lru, const char *key, uint key_len,
 		if (g->hashes[i] == khash_top) {
 			it = g->items[i];
 			if (likely(it && it->key_len == key_len
-					&& (key_len == 0 || memcmp(it->data, key, key_len) == 0)))
-				goto found; // to reduce huge nesting depth
+					&& (key_len == 0 || memcmp(it->data, key, key_len) == 0))) {
+				/* Found a key, but trying to insert a value larger than available
+				 * space in the allocated slot, so the entry must be resized to fit. */
+				if (unlikely(do_insert && val_len > it->val_len)) {
+					goto insert;
+				} else {
+					goto found; // to reduce huge nesting depth	
+				}
+			}
 		}
 	}
 	// key not found; first try an empty/counted-out place to insert
