@@ -102,17 +102,17 @@ typedef array_t(uint8_t) pack_t;
  *   for (uint8_t *it = pack_head(pack); it != pack_tail(pack); it = pack_obj_next(it))
  */
 #define pack_head(pack) \
-	(&(pack).at[0])
+	((pack).len > 0 ? &((pack).at[0]) : NULL)
 
 /** Return pack end pointer. */
 #define pack_tail(pack) \
-	(&(pack).at[(pack).len])
+	((pack).len > 0 ? &((pack).at[(pack).len]) : NULL)
 
 /** Return packed object length. */
 static inline pack_objlen_t pack_obj_len(uint8_t *it)
 {
 	pack_objlen_t len = 0;
-	if (it)
+	if (it != NULL)
 		memcpy(&len, it, sizeof(len));
 	return len;
 }
@@ -120,12 +120,20 @@ static inline pack_objlen_t pack_obj_len(uint8_t *it)
 /** Return packed object value. */
 static inline uint8_t *pack_obj_val(uint8_t *it)
 {
+	if (it == NULL) {
+		assert(it);
+		return NULL;
+	}
 	return it + sizeof(pack_objlen_t);
 }
 
 /** Return pointer to next packed object. */
 static inline uint8_t *pack_obj_next(uint8_t *it)
 {
+	if (it == NULL) {
+		assert(it);
+		return NULL;
+	}
 	return pack_obj_val(it) + pack_obj_len(it);
 }
 
@@ -160,7 +168,7 @@ static inline int pack_obj_push(pack_t *pack, const uint8_t *obj, pack_objlen_t
 		return kr_error(ENOSPC);
 	}
 
-	uint8_t *endp = pack_tail(*pack);
+	uint8_t *endp = pack->at + pack->len;
 	memcpy(endp, (char *)&len, sizeof(len));
 	memcpy(endp + sizeof(len), obj, len);
 	pack->len += packed_len;
@@ -173,7 +181,7 @@ static inline int pack_obj_push(pack_t *pack, const uint8_t *obj, pack_objlen_t
 static inline uint8_t *pack_obj_find(pack_t *pack, const uint8_t *obj, pack_objlen_t len)
 {
 		if (pack == NULL || obj == NULL) {
-			assert(false);
+			assert(obj != NULL);
 			return NULL;
 		}
 		uint8_t *endp = pack_tail(*pack);
@@ -194,7 +202,7 @@ static inline uint8_t *pack_obj_find(pack_t *pack, const uint8_t *obj, pack_objl
 static inline int pack_obj_del(pack_t *pack, const uint8_t *obj, pack_objlen_t len)
 {
 	if (pack == NULL || obj == NULL) {
-		assert(false);
+		assert(obj != NULL);
 		return kr_error(EINVAL);
 	}
 	uint8_t *endp = pack_tail(*pack);
