@@ -19,21 +19,22 @@
 #include "lib/defines.h"
 #include "lib/utils.h"
 
-#ifndef NOVERBOSELOG
- /** @internal Print a debug message related to resolution. */
- #define QRVERBOSE(query, cls, fmt, ...) { \
-    const struct kr_query *q = (query); \
-    if (kr_log_trace_enabled(q)) { \
-        kr_log_trace(q, cls, fmt, ##  __VA_ARGS__); \
-    } else WITH_VERBOSE(q) { \
-        unsigned _ind = 0; \
-        uint16_t _id = q ? q->id : 0; \
-        for (; q; q = q->parent, _ind += 2); \
-        kr_log_verbose("[%5hu][%s] %*s" fmt, _id, cls, _ind, "", ##  __VA_ARGS__); \
-    } \
-}
+#ifdef NOVERBOSELOG
+	#define QRVERBOSE(query, cls, ...)
 #else
- #define QRVERBOSE(query, cls, fmt, ...)
+	/** Print a debug message related to resolution.
+	 * \param _query	associated kr_query, may be NULL
+	 * \param _cls	identifying string, typically of length exactly four (padded)
+	 * \param ...	printf-compatible list of parameters
+	 */
+	#define QRVERBOSE(_query, _cls, ...) do { \
+		const struct kr_query *_qry = (_query); \
+		if (kr_log_trace_enabled(_qry)) { \
+			kr_log_trace(_qry, (_cls), __VA_ARGS__); \
+		} else if (VERBOSE_STATUS) { \
+			kr_log_qverbose_impl(_qry, (_cls), __VA_ARGS__); \
+		}  \
+	} while (false)
 #endif
 
 /** Layer processing states.  Only one value at a time (but see TODO).
@@ -83,8 +84,15 @@ struct kr_layer_api {
 	 * don't affect the resolution or rest of the processing. */
 	int (*checkout)(kr_layer_t *ctx, knot_pkt_t *packet, struct sockaddr *dst, int type);
 
+	/** Finalises the answer.
+	 * Last chance to affect what will get into the answer, including EDNS.*/
+	int (*answer_finalize)(kr_layer_t *ctx);
+
 	/** The module can store anything in here. */
 	void *data;
+
+	/** Internal to ./daemon/ffimodule.c. */
+	int cb_slots[];
 };
 
 typedef struct kr_layer_api kr_layer_api_t;
