@@ -23,7 +23,7 @@
 #include "lib/defines.h"
 #include "lib/layer.h"
 
-#define VERBOSE_MSG(qry, fmt...) QRVERBOSE(qry, "plan",  fmt)
+#define VERBOSE_MSG(qry, ...) QRVERBOSE(qry, "plan",  __VA_ARGS__)
 #define QUERY_PROVIDES(q, name, cls, type) \
     ((q)->sclass == (cls) && (q)->stype == type && knot_dname_is_equal((q)->sname, name))
 
@@ -179,7 +179,7 @@ static struct kr_query *kr_rplan_push_query(struct kr_rplan *rplan,
 	qry->timestamp_mono = kr_now();
 	qry->creation_time_mono = parent ? parent->creation_time_mono : qry->timestamp_mono;
 	kr_zonecut_init(&qry->zone_cut, (const uint8_t *)"", rplan->pool);
-	qry->reorder = qry->flags.REORDER_RR ? kr_rand_uint(0) : 0;
+	qry->reorder = qry->flags.REORDER_RR ? kr_rand_bytes(sizeof(qry->reorder)) : 0;
 
 	/* When forwarding, keep the nameserver addresses. */
 	if (parent && parent->flags.FORWARD && qry->flags.FORWARD) {
@@ -207,7 +207,8 @@ struct kr_query *kr_rplan_push_empty(struct kr_rplan *rplan, struct kr_query *pa
 	}
 
 	WITH_VERBOSE(qry) {
-	VERBOSE_MSG(qry, "plan '%s' type '%s'\n", "", "");
+	VERBOSE_MSG(qry, "plan '%s' type '%s'  uid [%05u.%02u]\n", "", "",
+		    qry->request ? qry->request->uid : 0, qry->uid);
 	}
 	return qry;
 }
@@ -230,7 +231,9 @@ struct kr_query *kr_rplan_push(struct kr_rplan *rplan, struct kr_query *parent,
 	WITH_VERBOSE(qry) {
 	KR_DNAME_GET_STR(name_str, name);
 	KR_RRTYPE_GET_STR(type_str, type);
-	VERBOSE_MSG(parent, "plan '%s' type '%s'\n", name_str, type_str);
+	VERBOSE_MSG(parent, "plan '%s' type '%s' uid [%05u.%02u]\n",
+		    name_str, type_str,
+		    qry->request ? qry->request->uid : 0, qry->uid);
 	}
 	return qry;
 }
