@@ -33,7 +33,7 @@
 #include "lib/rplan.h"
 #include "modules/cookies/cookiemonster.h"
 
-#define VERBOSE_MSG(qry, fmt...) QRVERBOSE(qry, "cookies",  fmt)
+#define VERBOSE_MSG(qry, ...) QRVERBOSE(qry, "cookies", __VA_ARGS__)
 
 /**
  * Obtain address from query/response context if if can be obtained.
@@ -415,7 +415,7 @@ int check_request(kr_layer_t *ctx)
 	};
 
 	struct kr_nonce_input nonce = {
-		.rand = kr_rand_uint(0),
+		.rand = kr_rand_bytes(sizeof(nonce.rand)),
 		.time = req->current_query->timestamp.tv_sec
 	};
 
