@@ -164,7 +164,9 @@ M.deinit = snapshot_end
 M.endpoints = {
 	['/stats']     = {'application/json', getstats, stream_stats},
 	['/frequent']  = {'application/json', function () return stats.frequent() end},
+	['/upstreams'] = {'application/json', function () return stats.upstreams() end},
+	['/bogus']     = {'application/json', function () return bogus_log.frequent() end},
 	['/metrics']   = {'text/plain; version=0.0.4', serve_prometheus},
 }
 
-return M
\ No newline at end of file
+return M
