@@ -1,6 +1,7 @@
 var colours = ["#081d58", "#253494", "#225ea8", "#1d91c0", "#41b6c4", "#7fcdbb", "#c7e9b4", "#edf8b1", "#edf8b1"];
 var latency = ["slow", "1500ms", "1000ms", "500ms", "250ms", "100ms", "50ms", "10ms", "1ms"];
 var Socket = "MozWebSocket" in window ? MozWebSocket : WebSocket;
+let isGraphPaused = false;
 
 $(function() {
 	/* Helper functions */
@@ -65,7 +66,7 @@ $(function() {
 		'worker.usertime':   [16, 'CPU (user)', null, 'Workers'],
 		'worker.systime':    [17, 'CPU (sys)', null, 'Workers'],
 	};
-	
+
 	/* Render latency metrics as sort of a heatmap */
 	var series = {};
 	for (var i in latency) {
@@ -81,7 +82,7 @@ $(function() {
 		labels.push(metrics[key][1]);
 		visibility.push(false);
 	}
-	
+
 	/* Define how graph looks like. */
 	const graphContainer = $('#stats');
     const graph = new Dygraph(
@@ -177,8 +178,10 @@ $(function() {
 		if (data.length > 1000) {
 			data.shift();
 		}
-		if (!buffer) {
-			graph.updateOptions( { 'file': data } );
+		if ( !buffer ) {
+			if ( !isGraphPaused ) {
+				graph.updateOptions( { 'file': data } );
+			}
 		}
 	}
 
@@ -223,7 +226,7 @@ $(function() {
 			}
 			/* Update bubble parameters */
 			if (!(key in found.name)) {
-				found.name.push(key);	
+				found.name.push(key);
 			}
 			found.rtt = (found.rtt + avg) / 2.0;
 			found.fillKey = colorBracket(found.rtt);
@@ -256,19 +259,19 @@ $(function() {
 			const stimeRate = updateRate(next.systime, data.last.systime, dt);
 			cell.eq(1).find('span').text(utimeRate + '% / ' + stimeRate + '%');
 			/* Update sparkline graph */
-			data.data.push([new Date(timestamp * 1000), utimeRate, stimeRate]);
+			data.data.push([new Date(timestamp * 1000), Number(utimeRate), Number(stimeRate)]);
 			if (data.data.length > 60) {
 				data.data.shift();
 			}
 			if (!buffer) {
-				data.graph.updateOptions( { 'file': data.data } );	
+				data.graph.updateOptions( { 'file': data.data } );
 			}
 		}
 		/* Update other fields */
 		if (!buffer) {
 			cell.eq(2).text(formatNumber(next.rss) + 'B');
 			cell.eq(3).text(next.pagefaults);
-			cell.eq(4).text('Healthy').addClass('text-success');	
+			cell.eq(4).text('Healthy').addClass('text-success');
 		}
 	}
 
@@ -349,4 +352,15 @@ $(function() {
 			pushMetrics(data.stats, data.time);
 		}
 	};
-});
\ No newline at end of file
+
+	chartElement.addEventListener( 'mouseover', ( event ) =>
+	{
+		isGraphPaused = true;
+	}, false );
+
+	chartElement.addEventListener( 'mouseout', ( event ) =>
+	{
+		isGraphPaused = false;
+	}, false );
+
+});
