@@ -81,12 +81,21 @@ end
 
 -- Override the list of nameservers (forwarders)
 local function set_nslist(qry, list)
-	for i, ns in ipairs(list) do
-		assert(ffi.C.kr_nsrep_set(qry, i - 1, ns) == 0);
+	local ns_i = 0
+	for _, ns in ipairs(list) do
+		-- kr_nsrep_set() can return kr_error(ENOENT), it's OK
+		if ffi.C.kr_nsrep_set(qry, ns_i, ns) == 0 then
+			ns_i = ns_i + 1
+		end
 	end
 	-- If less than maximum NSs, insert guard to terminate the list
-	if #list < 4 then
-		assert(ffi.C.kr_nsrep_set(qry, #list, nil) == 0);
+	if ns_i < 3 then
+		assert(ffi.C.kr_nsrep_set(qry, ns_i, nil) == 0);
+	end
+	if ns_i == 0 then
+		-- would use assert() but don't want to compose the message if not triggered
+		error('no usable address in NS set (check net.ipv4 and '
+		      .. 'net.ipv6 config):\n' .. table_print(list, 2))
 	end
 end
 
@@ -540,12 +549,18 @@ end
 -- as a dependency chain, e.g. r1,r2,r3 -> r3(r2(r1(state)))
 policy.layer = {
 	begin = function(state, req)
+		-- Don't act on "resolved" cases.
+		if bit.band(state, bit.bor(kres.FAIL, kres.DONE)) ~= 0 then return state end
+
 		req = kres.request_t(req)
 		return policy.evaluate(policy.rules, req, req:current(), state) or
 		       policy.evaluate(policy.special_names, req, req:current(), state) or
 		       state
 	end,
 	finish = function(state, req)
+		-- Don't act on "resolved" cases.
+		if bit.band(state, bit.bor(kres.FAIL, kres.DONE)) ~= 0 then return state end
+
 		req = kres.request_t(req)
 		return policy.evaluate(policy.postrules, req, req:current(), state) or state
 	end
