@@ -26,8 +26,10 @@ M.layer = {
 
 		local now = ffi.C.kr_now()
 		local deadline = qry.creation_time_mono + M.timeout
-		if now > deadline then
-			--log('[     ][stal]   => deadline has passed')
+		if now > deadline or qry.flags.NO_NS_FOUND then
+			if verbose() then
+				log('[     ][stal]   => no reachable NS, using stale data')
+			end
 			qry.stale_cb = M.callback
 			-- TODO: probably start the same request that doesn't stale-serve,
 			-- but first we need some detection of non-interactive / internal requests.
