@@ -11,8 +11,10 @@ package=knot-resolver
 if ! [[ "$1" == *-devel || "$1" == *-testing ]]; then
 	read -p "Pushing to '$project', are you sure? [y/N]: " yn
 	case $yn in
-		[Yy]* ) break;;
-		* ) exit 1; break;;
+		[Yy]* )
+            ;;
+		* )
+            exit 1
 	esac
 fi
 
