@@ -1,4 +1,5 @@
-#!/bin/bash -e
+#!/bin/bash
+set -o errexit -o nounset -o xtrace
 
 # Run with -s to include *.symbols files.
 
@@ -27,24 +28,26 @@ if [[ $(echo "${version}" | grep '^[[:alnum:].]$') -ne 0 ]]; then
 fi
 
 # Fill in VERSION field in distribution specific files
-files="distro/rpm/${package}.spec distro/deb/debian/changelog distro/deb/${package}.dsc distro/arch/PKGBUILD"
+files="distro/rpm/${package}.spec distro/deb/changelog distro/arch/PKGBUILD"
 for file in ${files}; do
 	sed -i "s/__VERSION__/${version}/g" "${file}"
 done
 
+# Rename archive to debian format
+pkgname="${package}-${version}"
+debname="${package}_${version}.orig"
+mv "${pkgname}.tar.xz" "${debname}.tar.xz"
+
+# Prepare clean debian-specific directory
+tar -xf "${debname}.tar.xz"
+pushd "${pkgname}" > /dev/null
+cp -arL ../distro/deb debian
+
 # Optionally remove symbols file
 if [ "$withsymbols" = false ]; then
-	rm distro/deb/debian/*.symbols
+    rm -f debian/*.symbols
 fi
 
-# Rename archive to debian format
-mv "${package}-${version}.tar.xz" "${package}_${version}.orig.tar.xz"
-
 # Create debian archive and dsc
-pushd distro/deb
-tar -chaf "${package}_${version}-1.debian.tar.xz" debian
-archive=${package}_${version}-1.debian.tar.xz
-echo " $(md5sum ${archive} | cut -d' ' -f1) $(wc -c ${archive})" >> ${package}.dsc
-popd
-archive=${package}_${version}.orig.tar.xz
-echo " $(md5sum ${archive} | cut -d' ' -f1) $(wc -c ${archive})" >> distro/deb/${package}.dsc
+dpkg-source -b .
+popd > /dev/null
