@@ -1,4 +1,5 @@
-#!/bin/bash -e
+#!/bin/bash
+set -o errexit -o nounset -o xtrace
 
 # Create a source rpm for Fedora/EPEL
 
@@ -7,6 +8,6 @@ scripts/make-archive.sh
 scripts/make-distrofiles.sh
 mv knot-resolver_*.orig.tar.xz distro/rpm/
 cd distro/rpm
-rpkg srpm
+rpkg srpm --outdir .
 mv *.src.rpm ../../
 mv *.tar.xz ../../
