@@ -1,9 +1,12 @@
 from collections import namedtuple, OrderedDict
 import glob
+import logging
 import os
 import re
 import yaml
 
+import pytest
+
 
 Scenario = namedtuple("Scenario", ["path", "qmin", "config"])
 
@@ -103,6 +106,8 @@ def rpls(paths):
 def pytest_addoption(parser):
     parser.addoption("--config", action="append", help="path to Deckard configuration .yaml file")
     parser.addoption("--scenarios", action="append", help="directory with .rpl files")
+    parser.addoption("--retries", action="store", help=("number of retries per"
+                                                        "test when Deckard is under load"))
 
 
 def pytest_generate_tests(metafunc):
@@ -125,3 +130,27 @@ def pytest_generate_tests(metafunc):
     if 'rpl_path' in metafunc.fixturenames:
         paths = metafunc.config.option.scenarios
         metafunc.parametrize("rpl_path", rpls(paths), ids=str)
+    if 'max_retries' in metafunc.fixturenames:
+        max_retries = metafunc.config.option.retries
+        if max_retries is None:
+            max_retries = 3
+        metafunc.parametrize("max_retries", [max_retries], ids=lambda id: "max-retries-"+str(id))
+
+
+def check_log_level_xdist(level):
+    if level < logging.ERROR:
+        pytest.exit("Advanced logging not available while running with xdist "
+                    "(try ommiting -n option)")
+
+
+def pytest_configure(config):
+    # This means pytest-xdist is installed and enabled
+    if hasattr(config.option, "dist") and config.option.dist == "load":
+        log_level = config.option.log_level
+        if log_level is None:
+            return
+        try:
+            log_level = int(log_level)
+        except ValueError:
+            log_level = logging.getLevelName(log_level)
+        check_log_level_xdist(log_level)
