@@ -754,6 +754,8 @@ class Step:
         answer = None
         if not self.data[0].is_raw_data_entry:
             while True:
+                if (datetime.now() - tstart).total_seconds() > 5:
+                    raise RuntimeError("Server took too long to respond")
                 try:
                     answer, _ = recvfrom_msg(sock, True)
                     break
@@ -896,7 +898,7 @@ class Scenario:
             for e in r.stored:
                 if e.mandatory and e.fired == 0:
                     # TODO: cisla radku
-                    raise RuntimeError('Mandatory section at %s not fired' % e.mandatory.span)
+                    raise ValueError('Mandatory section at %s not fired' % e.mandatory.span)
 
 
 def get_next(file_in, skip_empty=True):
