@@ -1,11 +1,9 @@
-from __future__ import absolute_import
-
 import argparse
 import itertools
 import logging
 import os
 import signal
-import select
+import selectors
 import socket
 import sys
 import threading
@@ -35,6 +33,7 @@ class TestServer:
         self.cur_iface = self.start_iface
         self.kroot_local = root_addr
         self.addr_family = addr_family
+        self.undefined_answers = 0
 
     def __del__(self):
         """ Cleanup after deletion. """
@@ -50,8 +49,8 @@ class TestServer:
                 raise Exception('TestServer already started')
         with self.active_lock:
             self.active = True
-        self.addr, _ = self.start_srv((self.kroot_local, port), self.addr_family)
-        self.start_srv(self.addr, self.addr_family, socket.IPPROTO_TCP)
+        addr, _ = self.start_srv((self.kroot_local, port), self.addr_family)
+        self.start_srv(addr, self.addr_family, socket.IPPROTO_TCP)
         self._bind_sockets()
 
     def stop(self):
@@ -92,25 +91,21 @@ class TestServer:
         if query is None:
             return False
         log.debug('server %s received query from %s: %s', server_addr, client_addr, query)
-        response, is_raw_data = self.scenario.reply(query, server_addr)
-        if response:
-            if not is_raw_data:
-                data_to_wire = response.to_wire(max_size=65535)
-                log.debug('response: %s', response)
-            else:
-                data_to_wire = response
-                log.debug('raw response not printed')
-        else:
-            response = dns.message.make_response(query)
-            response.set_rcode(dns.rcode.SERVFAIL)
-            data_to_wire = response.to_wire()
+
+        message = self.scenario.reply(query, server_addr)
+        if not message:
+            log.debug('ignoring')
+            return True
+        elif isinstance(message, scenario.DNSReplyServfail):
             self.undefined_answers += 1
             self.scenario.current_step.log.error(
                 'server %s has no response for question %s, answering with SERVFAIL',
                 server_addr,
                 '; '.join([str(rr) for rr in query.question]))
+        else:
+            log.debug('response: %s', message)
 
-        scenario.sendto_msg(client, data_to_wire, client_addr)
+        scenario.sendto_msg(client, message.to_wire(), client_addr)
         return True
 
     def query_io(self):
@@ -126,23 +121,30 @@ class TestServer:
                 if not self.active:
                     break
             objects = self.srv_socks + self.connections
-            to_read, _, to_error = select.select(objects, [], objects, 0.1)
-            for sock in to_read:
-                if sock in self.srv_socks:
-                    if sock.proto == socket.IPPROTO_TCP:
-                        conn, _ = sock.accept()
-                        self.connections.append(conn)
+            sel = selectors.DefaultSelector()
+            for obj in objects:
+                sel.register(obj, selectors.EVENT_READ)
+            items = sel.select(0.1)
+            for key, event in items:
+                sock = key.fileobj
+                if event & selectors.EVENT_READ:
+                    if sock in self.srv_socks:
+                        if sock.proto == socket.IPPROTO_TCP:
+                            conn, _ = sock.accept()
+                            self.connections.append(conn)
+                        else:
+                            self.handle_query(sock)
+                    elif sock in self.connections:
+                        if not self.handle_query(sock):
+                            sock.close()
+                            self.connections.remove(sock)
                     else:
-                        self.handle_query(sock)
-                elif sock in self.connections:
-                    if not self.handle_query(sock):
-                        sock.close()
-                        self.connections.remove(sock)
+                        raise Exception(
+                            "[query_io] Socket IO internal error {}, exit"
+                            .format(sock.getsockname()))
                 else:
-                    raise Exception(
-                        "[query_io] Socket IO internal error {}, exit".format(sock.getsockname()))
-            for sock in to_error:
-                raise Exception("[query_io] Socket IO error {}, exit".format(sock.getsockname()))
+                    raise Exception("[query_io] Socket IO error {}, exit"
+                                    .format(sock.getsockname()))
 
     def start_srv(self, address, family, proto=socket.IPPROTO_UDP):
         """ Starts listening thread if necessary """
@@ -259,13 +261,13 @@ def standalone_self_test():
 
     logging.info("[==========] Mirror server running at %s", server.address())
 
-    def exit(signum, frame):
+    def kill(signum, frame):  # pylint: disable=unused-argument
         logging.info("[==========] Shutdown.")
         server.stop()
         sys.exit(128 + signum)
 
-    signal.signal(signal.SIGINT, exit)
-    signal.signal(signal.SIGTERM, exit)
+    signal.signal(signal.SIGINT, kill)
+    signal.signal(signal.SIGTERM, kill)
 
     while True:
         time.sleep(0.5)
